#!/usr/bin/php
<?php

/**
  * Fix charset encoding of directories and files 
  * - converts to utf5
  * - fixes some garbled german umlauts 
  *
  * 2018-10-14 by klemens.ullmann-marx@ull.at
  */

// ARGUMENTS
if (!isset($argv[1])) {
	$script = basename($argv[0]);
	echo "Usage:\n    $script '/your/path' [--confirmation]\n";
}
$path = $argv[1];

$dryRun = true;
if (isset($argv[2]) && $argv[2] == '--confirmation') {
	$dryRun = false;
}

//$path = '/srv/nextcloud/klemens/files/Mullarx-Musik/Björk';
//$path = '/srv/nextcloud/klemens/files/Mullarx-Musik';
//$path = '/srv/nextcloud/klemens/files';

/*
hexdump('ü', false);
hexdump(hex2bin('75cc88'), false);
die;
*/

// FIX DIRS
$cmd = 'find ' . $path  . ' -type d' ;
$result = shell_exec($cmd);
$files = explode("\n", $result);
foreach ($files as $file) {
	fix($file);
}

// FIX FILES
$cmd = 'find ' . $path  . ' -type f' ;
$result = shell_exec($cmd);
$files = explode("\n", $result);
foreach ($files as $file) {
	fix($file);
}

echo "\n\n\n\n";



// FUNCTION 
function fix($file) {

	$tremas = array(
		hex2bin('61cc88') => 'ä',
		hex2bin('6Fcc88') => 'ö',
		hex2bin('75cc88') => 'ü',
		hex2bin('41cc88') => 'Ä',
		hex2bin('4Fcc88') => 'Ö',
		hex2bin('55cc88') => 'Ü',
	);

	// Special handling for directories
	$isDir = false;
	if (is_dir($file)) {
		$filename = $file;
		$isDir = true;
	} else {	
		$filename = basename($file);
	}
	
	//echo bin2hex($file) . "\n";
	//echo hexdump($filename, false);

	$questionableEncoding = !mb_check_encoding($filename, 'ASCII') && !mb_check_encoding($filename, 'UTF-8');
	$hasTremas = false;
	foreach ($tremas as $trema => $umlaut) {
		if (strstr($filename, $trema)) {
			$hasTremas = true;
			break;
		}
	}
	if (
	    $questionableEncoding 
	    || strstr($filename, ' (ungültige Kodierung)')
	    || $hasTremas
        ) {
		if ($isDir) {
			echo "############## DIR ##################\n";
		}		
		echo "--- " . $file . "\n";

		// CHARSET CONVERSION
		// apt install uchardet
		$cmd = 'echo "' . $filename . '" | uchardet';
//		apt install python-chardet
//		$cmd = 'echo "' . $filename . '" | chardetect';
		$charset = trim(shell_exec($cmd));
		if ($questionableEncoding) {
			echo 'Detected charset by uchardet: ' . $charset . "\n";
			$fixed = iconv($charset, 'UTF-8', $filename);
			echo 'Converted to UTF-8: ' . $fixed . "\n";
		} else {
			$fixed = $filename;
		}

		$garbled = array(
			'”' => 'ö',
			'„' => 'ä',
			'™' => 'Ö',
			'´' => '',
			'‚' => '', // not a normal comma!
			'' => 'ö',
			'' => 'ü',
			'á' => 'ß',
			'Ấ' => 'ä',
			'Ợ' => 'ö',
			
			' (ungültige Kodierung)' => '',
		) + $tremas;

		$isGarbled = false;
		foreach ($garbled as $garble => $replacement) {
			if (strstr($fixed, $garble)) {
				$isGarbled = true;
				$fixed = str_replace($garble, $replacement, $fixed);
				echo "Replaced garble chars '$garble' (" . bin2hex($garble) . ") into '$replacement' (". bin2hex($replacement) . ")\n";
			}
		}
		/* if ($isGarbled) {
			echo "Ungarbled: $fixed\n";
			hexdump($filename, false);
		} */

		//hexdump($filename, false);

		// NOW RENAME THE FILES
		$cmd = 'mv ' . escapeshellargMB($file) . ' ' . escapeshellargMB(str_replace($filename, $fixed, $file));
		echo $cmd . "\n";
		if ($dryRun) {
			echo $cmd . "\n";
		} else {
			echo shell_exec($cmd) . "\n";
		}

		echo "\n";
	}

}





// Do not remove multibyte chars like escapeshellarg ()
function escapeshellargMB($string) {
	$string = str_replace("'", "'\\''", $string);

	return "'$string'";
}


/**
 * View any string as a hexdump.
 *
 * This is most commonly used to view binary data from streams
 * or sockets while debugging, but can be used to view any string
 * with non-viewable characters.
 *
 * @version     1.3.2
 * @author      Aidan Lister <aidan@php.net>
 * @author      Peter Waller <iridum@php.net>
 * @link        http://aidanlister.com/2004/04/viewing-binary-data-as-a-hexdump-in-php/
 * @param       string  $data        The string to be dumped
 * @param       bool    $htmloutput  Set to false for non-HTML output
 * @param       bool    $uppercase   Set to true for uppercase hex
 * @param       bool    $return      Set to true to return the dump
 */
function hexdump ($data, $htmloutput = true, $uppercase = false, $return = false)
{
    // Init
    $hexi   = '';
    $ascii  = '';
    $dump   = ($htmloutput === true) ? '<pre>' : '';
    $offset = 0;
    $len    = strlen($data);
 
    // Upper or lower case hexadecimal
    $x = ($uppercase === false) ? 'x' : 'X';
 
    // Iterate string
    for ($i = $j = 0; $i < $len; $i++)
    {
        // Convert to hexidecimal
        $hexi .= sprintf("%02$x ", ord($data[$i]));
 
        // Replace non-viewable bytes with '.'
        if (ord($data[$i]) >= 32) {
            $ascii .= ($htmloutput === true) ?
                            htmlentities($data[$i]) :
                            $data[$i];
        } else {
            $ascii .= '.';
        }
 
        // Add extra column spacing
        if ($j === 7) {
            $hexi  .= ' ';
            $ascii .= ' ';
        }
 
        // Add row
        if (++$j === 16 || $i === $len - 1) {
            // Join the hexi / ascii output
            $dump .= sprintf("%04$x  %-49s  %s", $offset, $hexi, $ascii);
            
            // Reset vars
            $hexi   = $ascii = '';
            $offset += 16;
            $j      = 0;
            
            // Add newline            
            if ($i !== $len - 1) {
                $dump .= "\n";
            }
        }
    }
 
    // Finish dump
    $dump .= $htmloutput === true ?
                '</pre>' :
                '';
    $dump .= "\n";
 
    // Output method
    if ($return === false) {
        echo $dump;
    } else {
        return $dump;
    }
}



